**************************************************************
*  PROCEDURE PARSE
*  A routine to parse a memo field or other large character string
*  so that it fits between a predefined left and right margin.
*
*  It uses the MEMOTRAN() function to replace all of the soft carriage
*  returns with spaces and then searches backwards for spaces from
*  the end of the passed line length to properly support word wrap.
*
*  Syntax:
*    DO PARSE WITH <memo_field>, <left_tab>, <line_length>, <first_line>
*
*  Parameters:
*    memo_field  -  The name of the memofield to be parsed.
*    left_tab    -  The amount to indent from the left edge.
*    line_length  -  The maximum line length to print.
*    first_line  -  The indent for the first line of a paragraph.

*  Author: Ira Emus
*  Feb. 5, 1987
***************************************************************
*PROCEDURE PARSE  &&
PARAMETER MEMOFLD, LT_TAB, LINE_LENGTH, PAR_INDENT
                                                 ** MEMOFLD is the name of 
                                                 ** the memo field to print,
                                                 ** LT_TAB is the left indent,
                                                 ** LINE_LENGTH is the length
                                                 ** of the line to be printed 
                                                 ** and PAR_INDENT is the 
                                                 ** amount to indent the
                                                 ** first sentence of every
                                                 ** paragraph.

ADD_INDENT = .F.      && ADD_INDENT is the indent first line flag.
PARSE_LEN  =  0
MEMOSTR    =  MEMOTRAN(MEMOFLD, CHR(13), ' ')    && use MEMOTRAN() to 
                                                 ** replace all of
                                                 ** the soft cr's with
                                                 ** spaces the hard cr's
                                                 ** with CHR(13)
DO WHILE AT('  ', MEMOSTR) # 0
  MEMOSTR = SUBS(MEMOSTR,1,AT('  ',MEMOSTR))+SUBS(MEMOSTR,AT('  ',MEMOSTR)+2)
ENDDO                                            && remove extra spaces

IF PAR_INDENT > 0                                && the soft cr's with spaces
  MEMOSTR = SPACE(PAR_INDENT) + MEMOSTR          && the hard cr's with CHR(13)
ENDIF  &&

DO WHILE .T.
  IF LEN(MEMOSTR) > LINE_LENGTH
    PARSE_LEN = LINE_LENGTH
    IF AT(CHR(13), MEMOSTR) > PARSE_LEN .OR. AT(CHR(13), MEMOSTR) = 0    
                                                 ** test for absence of a
      DO WHILE SUBS(MEMOSTR, PARSE_LEN, 1) # ' ' && paragraph marker, and
        PARSE_LEN = PARSE_LEN - 1                && then locate a space to
      ENDDO                                      && split the line at.
    ELSE                                         && else split the line at
      PARSE_LEN = AT(CHR(13), MEMOSTR) - 1       && the paragraph mark.
      ADD_INDENT = .T.
    ENDIF
  ELSE
    IF AT(CHR(13), MEMOSTR) = 0                  && if MEMOSTR<LINE_LENGTH than
      PARSE_LEN = LEN(MEMOSTR)                   && check for hard cr in MEMOSTR
    ELSE                                         && IF FOUND SPLIT LINE, ELSE
      ADD_INDENT = .T.                           && print remainder of line
      PARSE_LEN = AT(CHR(13), MEMOSTR) - 1
    ENDIF
  ENDIF
  @ ROW() + 1, LT_TAB  SAY SUBS(MEMOSTR, 1, PARSE_LEN) ** print the string
                                                  ** on the next row
  PARSE_LEN = PARSE_LEN + 1                       && move past the end of
                                                  ** the previous line                                  
  DO WHILE SUBS(MEMOSTR, PARSE_LEN, 1) = ' ' .OR. ;
    SUBS(MEMOSTR, PARSE_LEN, 1) = CHR(13)
    IF SUBS(MEMOSTR, PARSE_LEN, 2) = CHR(13) + CHR(13)
      @ ROW() + 1, 1 SAY ''
    ENDIF
    PARSE_LEN = PARSE_LEN + 1                     && locate the next real
  ENDDO                                           && character
  MEMOSTR = SUBS(MEMOSTR, PARSE_LEN)              && delete the stuff we 
                                                  ** just printed
  IF ADD_INDENT .AND. PAR_INDENT > 0              && if this is the beginning
    ADD_INDENT = .F.                              && of a paragraph add the
    MEMOSTR = SPACE(PAR_INDENT) + MEMOSTR         && indent to the front of 
  ENDIF                                           && MEMOSTR.
  IF LEN(MEMOSTR) = 0                             && test for end of string
    EXIT                                          && and exit if reached
  ENDIF
ENDDO
RETURN

